/*
 *	ecvt converts to decimal
 *	the number of digits is specified by ndigit
 *	decpt is set to the position of the decimal point
 *	sign is set to 0 for positive, 1 for negative
 *
 *      Note that this deck contains some manifest floating point
 *      constants, which may produce problems when porting from
 *      one machine to another, in that the format of the floating
 *      constant value as produced by the compiler might not be
 *      compatible with the format on the target machine.  For this
 *      reason, if XBOOT is defined, no manifest constants are
 *      used, the floating value is generated at runtime.
 */

char	*cvt();

#define	NDIG	80

#ifdef XBOOT
# define ONE  one
# define TEN  ten
# define P03  p03
#else
# define ONE 1.0
# define TEN 10.0
# define P03 0.03
#endif

char*
ecvt(arg, ndigits, decpt, sign)
double arg;
int ndigits, *decpt, *sign;
{
	return(cvt(arg, ndigits, decpt, sign, 1));
}

char*
fcvt(arg, ndigits, decpt, sign)
double arg;
int ndigits, *decpt, *sign;
{
	return(cvt(arg, ndigits, decpt, sign, 0));
}

static char*
cvt(arg, ndigits, decpt, sign, eflag)
double arg;
int ndigits, *decpt, *sign;
{
	register int r2;
	double fi, fj;
	register char *p, *p1;
	static char buf[NDIG];
	double modf();
#ifdef XBOOT
	double one, ten, p03;
	int  i1, i2;

	i1 = 10;
	ten = i1;               /* ten = floating 10 on this machine */
	i1 = 1;
	one = i1;               /* one = floating 1 on this machine */
	i1 = 100;
	i2 = 3;
	p03 = (double)i2 / (double)i1;  /* p03 = .03 on this machine */
#endif

	if (ndigits<0)
		ndigits = 0;
	if (ndigits>=NDIG-1)
		ndigits = NDIG-2;
	r2 = 0;
	*sign = 0;
	p = &buf[0];
	if (arg<0) {
		*sign = 1;
		arg = -arg;
	}
	arg = modf(arg, &fi);
	p1 = &buf[NDIG];
	/*
	 * Do integer part
	 */
	if (fi != 0) {
		p1 = &buf[NDIG];
		while (fi != 0) {
			fj = modf(fi/TEN, &fi);
			*--p1 = (int)((fj+P03)*TEN) + '0';
			r2++;
		}
		while (p1 < &buf[NDIG])
			*p++ = *p1++;
	} else if (arg > 0) {
		while ((fj = arg*TEN) < ONE) {
			arg = fj;
			r2--;
		}
	}
	p1 = &buf[ndigits];
	if (eflag==0)
		p1 += r2;
	*decpt = r2;
	if (p1 < &buf[0]) {
		buf[0] = '\0';
		return(buf);
	}
	while (p<=p1 && p<&buf[NDIG]) {
		arg *= TEN;
		arg = modf(arg, &fj);
		*p++ = (int)fj + '0';
	}
	if (p1 >= &buf[NDIG]) {
		buf[NDIG-1] = '\0';
		return(buf);
	}
	p = p1;
	*p1 += 5;
	while (*p1 > '9') {
		*p1 = '0';
		if (p1>buf)
			++*--p1;
		else {
			*p1 = '1';
			(*decpt)++;
			if (eflag==0) {
				if (p>buf)
					*p = '0';
				p++;
			}
		}
	}
	*p = '\0';
	return(buf);
}
